Class: SleepyPenguin::Kevent

Inherits:
Struct
  • Object
show all
Defined in:
lib/sleepy_penguin/kevent.rb

Overview

This class represents a "struct kevent" structure for Ruby. This may be passed to Kqueue::IO#kevent as either a single element or as an element inside an array to inject changes into the kevent list.

Instance Attribute Summary collapse

Instance Attribute Details

#dataObject

Returns the value of attribute data

Returns:

  • (Object)

    the current value of data



5
6
7
# File 'lib/sleepy_penguin/kevent.rb', line 5

def data
  @data
end

#fflagsObject

Returns the value of attribute fflags

Returns:

  • (Object)

    the current value of fflags



5
6
7
# File 'lib/sleepy_penguin/kevent.rb', line 5

def fflags
  @fflags
end

#filterObject

Returns the value of attribute filter

Returns:

  • (Object)

    the current value of filter



5
6
7
# File 'lib/sleepy_penguin/kevent.rb', line 5

def filter
  @filter
end

#flagsObject

Returns the value of attribute flags

Returns:

  • (Object)

    the current value of flags



5
6
7
# File 'lib/sleepy_penguin/kevent.rb', line 5

def flags
  @flags
end

#identObject

Returns the value of attribute ident

Returns:

  • (Object)

    the current value of ident



5
6
7
# File 'lib/sleepy_penguin/kevent.rb', line 5

def ident
  @ident
end

#udataObject

Returns the value of attribute udata

Returns:

  • (Object)

    the current value of udata



5
6
7
# File 'lib/sleepy_penguin/kevent.rb', line 5

def udata
  @udata
end